#include <Application.h>
#include <InterfaceKit.h>
#include <StorageKit.h>
#include <TrackerAddon.h>
#include <UTF8.h>
#include <String.h>
#include <stdio.h>
#include <ctype.h>

int	HexToInt(const char *);
bool HexXToUTF8(char *, char *);
bool HexToUTF8(char *, char *);
bool ReplaceCapToUTF8(char *source, char *dest);



int	HexToInt(const char *hex)
{
int	i;
int	a = 1;
int	result = 0;
char	h;

	i = strlen( hex ) -1;
	for(;i>=0;i--){
		if (isxdigit(hex[i]) != 0) {
			h = toupper(hex[i]);
			switch (h) {
				case '0': break;
				case '1': result += 1 * a;
						break;
				case '2': result += 2 * a;
						break;
				case '3': result += 3 * a;
						break;
				case '4': result += 4 * a;
						break;
				case '5': result += 5 * a;
						break;
				case '6': result += 6 * a;
						break;
				case '7': result += 7 * a;
						break;
				case '8': result += 8 * a;
						break;
				case '9': result += 9 * a;
						break;
				case 'A': result += 0xa * a;
						break;
				case 'B': result += 0xb * a;
						break;
				case 'C': result += 0xc * a;
						break;
				case 'D': result += 0xd * a;
						break;
				case 'E': result += 0xe * a;
						break;
				case 'F': result += 0xf * a;
						break;
				default:  break;
			}
		} else {
			result = -1;
			break;
		};
		a = a * 0x10;
	};
	return result;
}

//	:xxX\0 形式のSJISをUTF8に変換
bool HexXToUTF8(char *source, char *dest)
{
char		caphex[3];
char		capstr[2];
int32	state = 0;
int32	sourceLength = 2;
int32	destinationLength = 3;

	if ( strlen(source) < 4 ) return false;
	if (( source[0] != 0x3a ) |
	    ( isxdigit(source[1]) == 0 ) |
	    ( isxdigit(source[2]) == 0 )) return false;

	caphex[2] = 0;

	caphex[0] = source[1];
	caphex[1] = source[2];
	capstr[0] = 0xff & HexToInt(caphex);
	capstr[1] = source[3];
	convert_to_utf8(B_SJIS_CONVERSION, capstr, &sourceLength, dest, &destinationLength, &state);
	return	true;
}

//	:xx:xx\0 形式のSJISをUTF8に変換
bool HexToUTF8(char *source, char *dest)
{
char		caphex[3];
char		capstr[2];
int32	state = 0;
int32	sourceLength = 2;
int32	destinationLength = 3;

	if ( strlen(source) < 6 ) return false;
	if (( source[0] != 0x3a ) |
	    ( source[3] != 0x3a ) |
	    ( isxdigit(source[1]) == 0 ) |
	    ( isxdigit(source[2]) == 0 ) |
	    ( isxdigit(source[4]) == 0 ) |
	    ( isxdigit(source[5]) == 0 )) {
		source[4] = 0;
		return HexXToUTF8(source, dest);
	};

	caphex[2] = 0;

	caphex[0] = source[1];
	caphex[1] = source[2];
	capstr[0] = 0xff & HexToInt(caphex);
	caphex[0] = source[4];
	caphex[1] = source[5];
	capstr[1] = 0xff & HexToInt(caphex);
	convert_to_utf8(B_SJIS_CONVERSION, capstr, &sourceLength, dest, &destinationLength, &state);
	return	true;
}

bool ReplaceCapToUTF8(char *source, char *dest)
{
char	buf[7];
char	nnm[4];
bool	result = false;

	buf[6] = 0;
	nnm[3] = 0;
	BString	filename(source);
	int i = filename.FindFirst(":");
//	printf("1: %d\n",i);
	while ( i != B_ERROR ) {
		for (int j=0;j<6;j++) buf[j] = filename.String()[i+j];
		if ( HexToUTF8(buf, nnm) ) {
//			printf("2: %s\n",filename.String());
//			printf("3: %s to %s\n",buf,nnm);
			filename.ReplaceAll(buf,nnm);
//			printf("4: %s\n",filename.String());
			result = true;
		} else i++;
		i = filename.FindFirst(":",i);
//		printf("5: %d\n",i);
	};
	sprintf(dest,"%s",filename.String());
	return	result;
}

void process_refs(entry_ref dir_ref, BMessage *msg, void *)
{
BEntry		target;
entry_ref		file_ref;
char			target_name[B_PATH_NAME_LENGTH];
char			new_name[B_PATH_NAME_LENGTH];

	for (int refs=0;msg->FindRef("refs", refs, &file_ref) == B_NO_ERROR;refs++) {
		target.SetTo(&file_ref);
		if (target.InitCheck() == B_NO_ERROR) {
			target.GetName(target_name);
			if ( ReplaceCapToUTF8(target_name, new_name) ) target.Rename(new_name);
			target.Unset();
		};
	};
}

int main()
{
	new BApplication("application/x-momoziro.CapRenamer");
	(new BAlert("", "CapRenamer Release1.0\n\n"
	                 "Samba cap形式の日本語ファイル名をUTF8にリネームするTracker Add-Onです。\n"
	                 "/boot/home/config/add-ons/Trackerに置いて使ってください。\n"
	                 "\n2000 momoziro研究所\n"
	                 " http://www.be-in.org/~momoziro/index.html\n"
	                 " mailto:momoziro@be-in.org", "OK"))->Go();
	delete be_app;
}
